using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Web;
using System.Web.Services;
using UJ.ExceptionManagement;

namespace UJ.Demos.EmsDemoWebService
{
	/// <summary>
	/// Summary description for Service1.
	/// </summary>
	public class EmsDemoWebService : System.Web.Services.WebService	{
		public EmsDemoWebService()
		{
			//CODEGEN: This call is required by the ASP.NET Web Services Designer
			InitializeComponent();
		}

		#region Component Designer generated code
		
		//Required by the Web Services Designer 
		private IContainer components = null;
				
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{

    }

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if(disposing && components != null)
			{
				components.Dispose();
			}
			base.Dispose(disposing);		
		}
		
		#endregion

    //See initialization code for Web Service Global.asax file, Application_Start method 
    [WebMethod]
    public string HelloWorld(int action, bool serverDebug)	{
    	try{
        Ems.Settings.DebugBuild = serverDebug;
        switch(action) {
          case 0: //return normally
            return "Hello World!";
          case 1: //Action - throwing validation exception
           /*
           Let's pretend we validate the request data, validation fails, and we throw a validation exception.
           EMS will handle the ValidationException in the catch block in WebMethod , and rethrow it as a SoapException 
           with Code property set to ClientFault. This code would tell the client that the request is invalid. 
           No exception report will be produced on WebService side in this case, as there is no WebService failure. 
           EMS would put some additional information into the Details property of the SoapException object 
           - the failed method name, the exception message, and names/values of the parameters. 
           This information would help to investigate the failure on the client.
           When client app receives the SoapException with ClientFault code, it must be interpreted 
           as a FATAL exception (not a validation fault - the client App had made an invalid request, 
           that should never happen, so the client App should be fixed). In this demo EMS 
           would produce the exception report on the client side about the failure. 
           */            
            Ems.Validate(false, "DemoWebService: Invalid request data. "); //that will throw ValidationException
            break;
          case 2: //Action - emulating server error; 
            /*
            This is the case of the internal server failure. EMS would handle a fatal exception in the catch block, 
            process it, build and publish the exception report right on the server side. EMS will 
            also rethrow the exception as a SoapException with the Code property set to ServerFault in order 
            to let the client know about the failure, and fill the Detail element of the SoapException. 
            The additional information in the Details element of a SoapException depends on the service build type. 
            For debug builds, EMS would put the entire exception report into the SoapException, just to help the developer 
            - who is probably debugging both server and client applications - to investigate the failure. 
            For release builds, EMS puts only a brief message about the error  into the Detail element. 
            The message is constructed using the Ems.Settings.UserMessageTemplate template.
            Now about client application behavior on receiving a SoapException with Code=ServerFault. 
            Service client normally should not interpret ServerFault as its own (client's) fatal exception. Fatal exception 
            would result in exception report on the client, but there is nothing wrong with the client App in this case - 
            it is a server fault. 
            EMS helps the client app to implement such a "not-my-fault" behavior by (optionally) transforming
            the remote server fault into RemoteServerException. The RemoteServerException class is defined by EMS
            as a descendant of the ValidationException, so as all other validation exceptions, this exception would result 
            in a message box, without producing the exception report or generating emergency messages to a developer. 
            This SoapException->RemoteException wrapping is done automatically by EMS. It is turned on/off 
            by the Ems.Settings.ClientSoapWrapMode property. 
            */            
            throw new ApplicationException("Internal service error."); 
        }
        return "";
    	}//try
    	catch(Exception _ex){
        // With the action parameter set to WebMethod the HandleException method would rethrow SoapException.
        // It will copy 
    		Ems.HandleException(this, _ex, HandleAction.WebMethod, Ems.AsParams(action, serverDebug));
    		throw;
    	}//catch
    }

    //We use static constructor to initialize EMS. You could also use the Application_Start method in Global.asax file.
    static EmsDemoWebService() {
#if DEBUG
      InitHelper.InitializeForWebService(true);   
#else
      InitHelper.InitializeForWebService(false);   
#endif
      Ems.Settings.UserMessageTemplate = "Internal Service Error. Please try again later.";
    }
  }//class
}//namespace
